VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "MapRegion"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public PtX As New Collection
Public PtY As New Collection
Public Neighbors As Collection
Public Color As Integer
Public Traversed As Boolean

Public Number As Integer    ' Our number.

Private NeighborsCopy As Collection

' Assign the color to this region. Then recursively
' assign the other colors to our neighbors.
' Return True if there is an error.
Public Function Assign2Color(ByVal clr As Integer)
Dim nbr As MapRegion

    ' See if we already have the right color.
    If Color = clr Then
        ' We already have the right color.
        ' Return a success code.
        Assign2Color = False
        Exit Function
    End If

    ' See if we have the wrong color.
    If Color <> UNCOLORED Then
        ' We have the wrong color.
        ' Return an error code.
        Assign2Color = True
        Exit Function
    End If

    ' Assign the color.
    Color = clr

    ' Assign colors to our neighbors.
    clr = 1 - clr
    For Each nbr In Neighbors
        If nbr.Assign2Color(clr) Then
            ' There was an error.
            Assign2Color = True
            Exit Function
        End If
    Next nbr

    ' Return a success code.
    Assign2Color = False
End Function

' Return True if one of the neighbors
' is using this color.
Public Function ColorUsedByNeighbors(ByVal clr As Integer) As Boolean
Dim nbr As MapRegion

    ' Assume a neighbor is using the color.
    ColorUsedByNeighbors = True

    ' Check the neighbors.
    For Each nbr In Neighbors
        If nbr.Color = clr Then Exit Function
    Next nbr

    ' No neighbor is using the color.
    ColorUsedByNeighbors = False
End Function

' Return True if this region has a different
' color than its neighbors.
Public Function VerifyColoring(ByRef MAX_COLOR As Integer) As Boolean
    ' Assume this is not valid.
    VerifyColoring = False

    ' See if we have a color.
    If Color = UNCOLORED Then Exit Function

    ' See if a neighbor has our color.
    If ColorUsedByNeighbors(Color) Then Exit Function

    ' This is valid.
    VerifyColoring = True
End Function

' Recursively traverse this region and its neighbors.
Public Sub Traverse(ByVal traversal As Collection)
    ' Do nothing if we have already been traversed.
    If Traversed Then Exit Sub

    ' Add ourself to the traversal.
    Traversed = True
    traversal.Add Me

    ' Traverse our neighbors.
    Dim nbr As MapRegion
    For Each nbr In Neighbors
        nbr.Traverse traversal
    Next nbr
End Sub

' See if rgn is a neighbor of ours.
Public Sub CheckNeighbor(rgn As MapRegion)
Dim i As Integer
Dim x1 As Single
Dim y1 As Single
Dim x2 As Single
Dim y2 As Single

    x1 = PtX(1)
    y1 = PtY(1)
    For i = 2 To PtX.count
        x2 = PtX(i)
        y2 = PtY(i)
        If rgn.HasLine(x1, y1, x2, y2) Then
            ' Howdy neighbor!
            Neighbors.Add rgn, Format$(rgn.Number)
            rgn.Neighbors.Add Me, Format$(Number)
            Exit Sub
        End If
        x1 = x2
        y1 = y2
    Next i
End Sub
' Identify this region with rgn. Copy this
' node's neighbors into rgn's neighbor list.
Public Sub AssociateWith(rgn As MapRegion)
Dim n1 As MapRegion
Dim numtxt As String

    numtxt = Format$(rgn.Number)
    For Each n1 In Neighbors
        If Not (n1 Is rgn) Then
            On Error Resume Next
            
            ' Add the neighbor to rgn's list.
            rgn.Neighbors.Add n1, Format$(n1.Number)
            If Err.Number = 0 Then
                ' It worked. Add rgn to n1's list.
                n1.Neighbors.Add rgn, numtxt
            End If
            
            On Error GoTo 0
        End If
    Next n1
End Sub

' Assign this region the first color number
' not used by a neighbor and return the number.
Public Function GreedyColor() As Integer
Dim nbr As MapRegion
Dim color_used(-1 To 10) As Boolean

    For Each nbr In Neighbors
        
    Next nbr
End Function

' Find the smallest color number that is not
' used by our neighbors.
Public Sub PickUnusedColor()
Dim color_used(-1 To 4) As Boolean
Dim nbr As MapRegion
Dim i As Integer

    ' Do nothing if we are already colored.
    If Color <> UNCOLORED Then Exit Sub

    ' Mark the neighbors' colors as used.
    For Each nbr In Neighbors
        color_used(nbr.Color) = True
    Next nbr

    ' Assume we won't find an unused color.
    Color = UNCOLORED

    ' Find the smallest unused color.
    For i = 0 To MAX_COLOR
        If Not color_used(i) Then
            Color = i
            Exit For
        End If
    Next i
End Sub
' Remove the node from the graph.
Public Sub RemoveFromGraph()
Dim rgn As MapRegion
Dim numtxt As String

    ' Remove links from neighbors to here.
    numtxt = Format$(Number)
    For Each rgn In Neighbors
        rgn.Neighbors.Remove numtxt
    Next rgn

    ' Remove links to neighbors.
    Set Neighbors = Nothing
End Sub
' Save the neighbor list for later.
Public Sub SaveNeighbors()
Dim rgn As MapRegion

    Set NeighborsCopy = New Collection
    For Each rgn In Neighbors
        NeighborsCopy.Add rgn
    Next rgn
End Sub
' Restore the saved neighbor list.
Public Sub RestoreNeighbors()
Dim rgn As MapRegion

    Set Neighbors = New Collection
    For Each rgn In NeighborsCopy
        Neighbors.Add rgn, Format$(rgn.Number)
    Next rgn
    Set NeighborsCopy = Nothing
End Sub
' Return true if this region includes the line.
Public Function HasLine(a1 As Single, b1 As Single, a2 As Single, b2 As Single) As Boolean
Dim i As Integer
Dim x1 As Single
Dim y1 As Single
Dim x2 As Single
Dim y2 As Single

    ' Assume we will find it.
    HasLine = True
    
    x1 = PtX(1)
    y1 = PtY(1)
    For i = 2 To PtX.count
        x2 = PtX(i)
        y2 = PtY(i)
        If x1 = a1 And y1 = b1 And _
           x2 = a2 And y2 = b2 _
                Then Exit Function
        If x2 = a1 And y2 = b1 And _
           x1 = a2 And y1 = b2 _
                Then Exit Function
        x1 = x2
        y1 = y2
    Next i
    
    ' We didn't find the line.
    HasLine = False
End Function


' Display the text as the approximate center of
' the region.
Public Sub ShowAtCenter(ctl As PictureBox, txt As String)
Dim i As Integer
Dim totx As Single
Dim toty As Single

    ' Find the approximate center.
    totx = 0
    toty = 0
    For i = 1 To PtX.count
        totx = totx + PtX(i)
        toty = toty + PtY(i)
    Next i
    totx = totx / PtX.count
    toty = toty / PtX.count

    ' Display the text.
    ctl.CurrentX = totx - ctl.TextWidth(txt) / 2
    ctl.CurrentY = toty - ctl.TextHeight(txt) / 2
    ctl.Print txt
End Sub

' Draw the region.
Public Sub DrawRegion(ctl As PictureBox)
Dim pts() As POINTAPI
Dim i As Integer

    ReDim pts(1 To PtX.count)
    For i = 1 To PtX.count
        pts(i).X = PtX(i)
        pts(i).Y = PtY(i)
    Next i

    ctl.FillColor = Colors(Color)
    Polygon ctl.hdc, pts(1), PtX.count
End Sub


' Find two mutually non-adjacent neighbors with
' degree <= 7.
Public Function FindNonAdjacents(n1 As MapRegion, n2 As MapRegion) As Boolean
Dim n3 As MapRegion
Dim non_adjacent As Boolean

    FindNonAdjacents = True
    For Each n1 In Neighbors
        ' See if n1 has degree <= 7.
        If n1.Neighbors.count <= 7 Then
            For Each n2 In Neighbors
                If Not (n1 Is n2) Then
                    ' See if n2 has degree <= 7.
                    If n2.Neighbors.count <= 7 Then
                        ' See if n1 and n2 are
                        ' non-adjacent.
                        non_adjacent = True
                        For Each n3 In n1.Neighbors
                            If n3 Is n2 Then
                                non_adjacent = False
                                Exit For
                            End If
                        Next n3
                        If non_adjacent Then Exit Function
                    End If ' End if Degree(n2) <= 7.
                End If ' End if n1 <> n2.
            Next n2
        End If ' End if Degree(n1) <= 7.
    Next n1
    FindNonAdjacents = False
End Function
' List the neighbors.
Public Sub ListNeighbors()
Dim rgn As MapRegion

    Debug.Print Format$(Number, "@@@") & ": ";
    For Each rgn In Neighbors
        Debug.Print Format$(rgn.Number, "@@@@");
    Next rgn
    Debug.Print
End Sub


' Display the neighbor count.
Public Sub ShowCount(ctl As PictureBox)
    ShowAtCenter ctl, Format$(Neighbors.count)
End Sub
' Display the region number.
Public Sub ShowNumber(ctl As PictureBox)
    ShowAtCenter ctl, Format$(Number)
End Sub

' If there is an existing point within SNAP
' pixels in the X and Y directions, use its
' coordinates.
Public Sub SnapTo(X As Single, Y As Single)
Const SNAP = 5
Dim i As Integer

    For i = 1 To PtX.count
        If Abs(X - PtX(i)) < SNAP And _
           Abs(Y - PtY(i)) < SNAP _
        Then
            X = PtX(i)
            Y = PtY(i)
            Exit Sub
        End If
    Next i
End Sub


' Set the control's default color.
Public Sub Uncolor()
    Color = UNCOLORED
End Sub
' Load the region's data from a file.
Public Sub LoadData(fnum As Integer)
Dim num_points As Integer
Dim X As Single
Dim Y As Single
Dim i As Integer

    Input #fnum, num_points
    For i = 1 To num_points
        Input #fnum, X, Y
        PtX.Add X
        PtY.Add Y
    Next i
End Sub
' Save the region's data into a file.
Public Sub SaveData(fnum As Integer)
Dim i As Integer

    Write #fnum, PtX.count
    For i = 1 To PtX.count
        Write #fnum, PtX(i), PtY(i)
    Next i
End Sub

' Set the control's default color.
Private Sub Class_Initialize()
    Uncolor
End Sub
